import 'dart:developer';

import 'package:dyte_core/dyte_core.dart';
import 'package:dyte_core_example/constants/colors.dart';
import 'package:dyte_core_example/models/meeting_details.dart';
import 'package:dyte_core_example/models/participant_details.dart';
import 'package:dyte_core_example/models/states/room_event_states.dart';
import 'package:dyte_core_example/providers/providers.dart';
import 'package:dyte_core_example/repository/meeting_handler.dart';
import 'package:dyte_core_example/ui/meeting/staging_room.dart';
import 'package:dyte_core_example/ui/widgets/appbar.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

class HomePage extends ConsumerStatefulWidget {
  const HomePage({super.key});

  @override
  ConsumerState<HomePage> createState() => _HomePageState();
}

class _HomePageState extends ConsumerState<HomePage> {
  final nameController = TextEditingController();

  final descriptionController = TextEditingController();

  final roomNameController = TextEditingController();

  final meetingRepo = MeetingRepository();

  MeetingDetails? meetingDetails;
  ParticipantTokens? participantTokens;

  @override
  Widget build(BuildContext context) {
    ref.listen<RoomEventStates>(roomEventStateNotifierProvider, (_, next) {
      next.maybeWhen(
        onMeetingInitFailed: (exception) =>
            log("MEETING INIT FAILED EXCEPTION: ${exception.toString()}"),
        onMeetingInitStarted: () => log("MEETING INIT STARTED!!"),
        onMeetingInitCompleted: () {
          Navigator.push(context, MaterialPageRoute(builder: (context) {
            return const StagingRoom();
          }));
        },
        orElse: () {},
      );
    });
    return Scaffold(
      appBar: const DyteAppBar(),
      body: Center(child: _meetingDetails('wsyeyy-uirzla')),
    );
    // return Scaffold(
    //   body: SafeArea(
    //     child: SingleChildScrollView(
    //       child: SizedBox(
    //         height: MediaQuery.of(context).size.height,
    //         child: Column(
    //           mainAxisAlignment: MainAxisAlignment.start,
    //           crossAxisAlignment: CrossAxisAlignment.center,
    //           children: [
    //             const SizedBox(height: 10),
    //             SvgPicture.asset(
    //               'assets/dyte_logo.svg',
    //               width: 100,
    //               height: 50,
    //             ),
    //             const SizedBox(height: 20),
    //             const Text(
    //               "Create Meeting",
    //               style: TextStyle(
    //                 fontSize: 24,
    //                 fontWeight: FontWeight.w500,
    //               ),
    //             ),
    //             const SizedBox(height: 10),
    //             Padding(
    //               padding:
    //                   const EdgeInsets.symmetric(vertical: 12, horizontal: 16),
    //               child: _textField(nameController, 'Your Name'),
    //             ),
    //             Divider(
    //               color: Colors.grey.shade300,
    //               thickness: 1,
    //             ),
    //             Padding(
    //               padding:
    //                   const EdgeInsets.symmetric(vertical: 6, horizontal: 16),
    //               child: _textField(
    //                   descriptionController, 'What\'s your meeting about?'),
    //             ),
    //             const SizedBox(height: 10),
    // MaterialButton(
    //   onPressed: () async {
    //     if (nameController.text.isNotEmpty) {
    //       _createMeetingAndAddMe(nameController.text);
    //     }
    //   },
    //   minWidth: 180,
    //   height: 45,
    //   color: DyteColors.dytePrimary,
    //   child: const Text(
    //     "Create Meeting",
    //     style: TextStyle(fontSize: 16),
    //   ),
    // ),
    //             const SizedBox(height: 10),
    //             Padding(
    //               padding:
    //                   const EdgeInsets.symmetric(vertical: 6, horizontal: 16),
    //               child: _textField(roomNameController, 'Room Name'),
    //             ),
    //             const SizedBox(height: 10),
    //             MaterialButton(
    //               onPressed: () async {
    //                 if (nameController.text.isNotEmpty &&
    //                     roomNameController.text.isNotEmpty) {
    //                   _searchRoomAndAddMe(
    //                       nameController.text, roomNameController.text);
    //                 }
    //               },
    //               minWidth: 180,
    //               height: 45,
    //               color: DyteColors.dytePrimary,
    //               child: const Text(
    //                 "Join Room",
    //                 style: TextStyle(fontSize: 16),
    //               ),
    //             ),
    //             const Spacer(),
    //             if (participantTokens != null)
    //               _meetingDetails(meetingDetails!.data.meeting.roomName),
    //             const Spacer(),
    //           ],
    //         ),
    //       ),
    //     ),
    //   ),
    // );
  }

  Future<void> _searchRoomAndAddMe(
      String participantName, String roomName) async {
    final meetingId = await meetingRepo.getMeetingIdFromRoomName(roomName);
    if (meetingId != null) {
      participantTokens =
          await meetingRepo.addAParticipant(participantName, meetingId);
      log(participantTokens!.message);
    }
    setState(() {});
  }

  Future<void> _addLocalUser(
      String participantName, MeetingDetails? meetingDetails) async {
    if (meetingDetails != null) {
      participantTokens = await meetingRepo.addAParticipant(
          participantName, meetingDetails.data.meeting.id);
      log(participantTokens!.message);
    }
    setState(() {});
  }

  Future<void> _createMeetingAndAddMe(String participantName) async {
    meetingDetails = await meetingRepo.createMeeting();
    _addLocalUser(participantName, meetingDetails);
  }

  Widget _meetingDetails(String meetLink) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        GestureDetector(
          onTap: () {
            Clipboard.setData(
                ClipboardData(text: 'https://app.dyte.io/$meetLink'));
          },
          child: Text('https://app.dyte.io/$meetLink'),
        ),
        const SizedBox(height: 10),
        MaterialButton(
          onPressed: () {
            ref.watch(dyteClient).addMeetingRoomEventsListener(
                ref.watch(roomEventStateNotifierProvider.notifier));
            ref.watch(dyteClient).addParticipantEventsListener(
                ref.watch(participantEventStateNotifierProvider.notifier));
            ref
                .watch(dyteClient)
                .addSelfEventsListener(ref.watch(localUserProvider.notifier));

            ref.watch(dyteClient).init(
                  DyteMeetingInfo(
                    roomName: "wsyeyy-uirzla",
                    enableAudio: true,
                    displayName: 'Flutter core',
                    authToken:
                        'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjY1YjFlYjJmLTA4OGEtNDdkMi1iOWQ2LTdhN2U5ODY3OTQ5YiIsImxvZ2dlZEluIjp0cnVlLCJpYXQiOjE2NjQ4NjU1NjYsImV4cCI6MTY3MzUwNTU2Nn0.QiSnsvKF6xN0hDzkM4T3oVvp66MuE-oBK1SYkPXcKb4uhK3uMoIsfqNyVP6jsbw_3D5Hse5dW8tHTxCO3xzzqNYzUVA9rM8rMGiunYUVTs64QJ2SiPLSZr9myfmsWyRyjseRoRnwFcso7zr8yY1ZxyV_bwq3QbsJsrJH5rm9Dt_X42Qk-c4Ass_a7vFbqS65n3vrY78xAB33A9-4Gkvwvy9j-ZH3bvHojgMyx3dmAK7T2kmQgnlwtn3l3CaxexnmCyu9-ow5U50IFzBvHFkCabQKEuY5Qf8HBM0OmSZZcuBBxSDrn3eV2Jc0QbwyHGTqLVO9Mk9f4pN9TOI3gPomYA',
                    orgId: "60709d2a-c83e-477a-8199-f00ff680c44d",
                  ),
                );
          },
          minWidth: 180,
          height: 45,
          color: DyteColors.primary,
          child: const Text(
            "Join Meeting",
            style: TextStyle(fontSize: 16),
          ),
        )
      ],
    );
  }

  Widget _textField(TextEditingController controller, String hintText) {
    return TextField(
      controller: controller,
      decoration: InputDecoration(
          focusedBorder: OutlineInputBorder(
              borderSide: BorderSide(color: Colors.grey.shade200)),
          border: const OutlineInputBorder(),
          hintText: hintText),
    );
  }
}
