import 'package:dyte_core/dyte_core.dart';
import 'package:dyte_core_example/constants/colors.dart';
import 'package:dyte_core_example/providers/providers.dart';
import 'package:dyte_core_example/ui/home_page.dart';
import 'package:dyte_core_example/ui/widgets/appbar.dart';
import 'package:dyte_core_example/ui/widgets/dyte_button.dart';
import 'package:dyte_core_example/ui/widgets/dyte_call_bottom_bar.dart';
import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

class DyteRoom extends ConsumerStatefulWidget {
  const DyteRoom({Key? key}) : super(key: key);

  @override
  ConsumerState<DyteRoom> createState() => _DyteRoomState();
}

class _DyteRoomState extends ConsumerState<DyteRoom> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final size = MediaQuery.of(context).size;
    return Scaffold(
      appBar: DyteAppBar(
        action: DyteButton(
          label: 'Leave',
          size: const Size(80, 30),
          color: DyteColors.red,
          onPressed: () {
            ref.watch(dyteClient).leaveRoom();
            Navigator.pushAndRemoveUntil(
              context,
              MaterialPageRoute(
                builder: (context) => const HomePage(),
              ),
              (route) => false,
            );
          },
        ),
      ),
      bottomNavigationBar: const DyteCallBottomBar(),
      body: _renderMeetingViewStreamBuilder(size),
    );
  }

  Widget _renderMeetingViewStreamBuilder(Size size) {
    return StreamBuilder<DyteRoomParticipants>(
      stream: ref.read(dyteClient).participantsStream,
      initialData: ref.watch(dyteClient).participants,
      builder: (context, snapshot) {
        final data = snapshot.data;
        if (data != null) {
          return Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Text(
                'Participants: ${data.joined.length}',
                style: const TextStyle(
                  color: Colors.white,
                  fontWeight: FontWeight.bold,
                  fontSize: 18,
                ),
              ),
              const SizedBox(height: 10),
              SizedBox(
                height: size.height * .75,
                child: _renderVideoGrid(data.joined),
              ),
              // const Spacer(),
            ],
          );
        }
        return const Center(
          child: CircularProgressIndicator(),
        );
      },
    );
  }

  Widget _renderVideoGrid(List<DyteMeetingParticipant> participants) {
    return GridView(
      gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
        crossAxisCount: 2,
        childAspectRatio: 1,
        crossAxisSpacing: 10,
        mainAxisSpacing: 10,
        mainAxisExtent: 200,
      ),
      children: participants.map((e) {
        return VideoView(e);
      }).toList(),
    );
  }
}
